import React from 'react';
import withStyles from '@material-ui/styles/withStyles';
import FormControl from '@material-ui/core/FormControl';
import InputLabel from '@material-ui/core/InputLabel'; // import FormHelperText from '@material-ui/core/FormHelperText'

import draftToHtml from 'draftjs-to-html';
import { CompositeDecorator, ContentState, Editor, EditorState, convertFromHTML, convertToRaw, RichUtils } from 'draft-js';
import BlockStyleControls from './BlockStyleControls';
import InlineStyleControls from './InlineStyleControls';
import styles from './richTextareaStyles';

class RichTextarea extends React.Component {
  constructor(props) {
    super(props);
    const decorator = new CompositeDecorator([{
      strategy: findLinkEntities,
      component: Link
    }, {
      strategy: findImageEntities,
      component: Image
    }]);
    let content = null;

    if (props.value) {
      const blocksFromHTML = convertFromHTML(props.value);
      content = ContentState.createFromBlockArray(blocksFromHTML.contentBlocks, blocksFromHTML.entityMap);
    }

    this.editor = null;
    this.state = {
      editorState: content ? EditorState.createWithContent(content, decorator) : EditorState.createEmpty(decorator),
      hasFocus: false
    };
    this.onChange = this.onChange.bind(this);
    this.onBlur = this.onBlur.bind(this);

    this.focus = () => {
      this.setState({
        hasFocus: true
      });
      return this.editor.focus();
    };

    this.handleKeyCommand = this._handleKeyCommand.bind(this);
    this.onTab = this._onTab.bind(this);
    this.toggleBlockType = this._toggleBlockType.bind(this);
    this.toggleInlineStyle = this._toggleInlineStyle.bind(this);
  }

  onBlur(editorState) {
    this.setState({
      hasFocus: false
    });
    this.setState({
      editorState
    });
  }

  onChange(editorState) {
    const rawContentState = convertToRaw(editorState.getCurrentContent());
    let markup = draftToHtml(rawContentState); // Check for empty:

    if (markup.replace(/(<([^>]+)>)|\n|\r|\t/ig, '').length === 0) {
      markup = '';
    }

    if (this.props.onChange) {
      this.props.onChange(markup);
    }

    this.setState({
      editorState
    });
  }

  _handleKeyCommand(command, editorState) {
    const newState = RichUtils.handleKeyCommand(editorState, command);

    if (newState) {
      this.onChange(newState);
      return true;
    }

    return false;
  }

  _onTab(e) {
    const maxDepth = 2;
    this.onChange(RichUtils.onTab(e, this.state.editorState, maxDepth));
  }

  _toggleBlockType(blockType) {
    this.focus();
    this.onChange(RichUtils.toggleBlockType(this.state.editorState, blockType));
  }

  _toggleInlineStyle(inlineStyle) {
    this.onChange(RichUtils.toggleInlineStyle(this.state.editorState, inlineStyle));
  }

  render() {
    const {
      editorState,
      hasFocus
    } = this.state;
    const {
      t,
      classes,
      value,
      label,
      disabled,
      style,
      shrink,
      underline
    } = this.props;
    return React.createElement(FormControl, {
      style: style,
      className: `${classes.richRoot}`,
      onFocus: this.focus,
      fullWidth: true
    }, React.createElement(InputLabel, {
      htmlFor: `${label}-richtextarea`,
      shrink: hasFocus || !!value && !hasFocus || shrink,
      onClick: this.focus
    }, label), React.createElement("div", {
      role: "button",
      tabIndex: 0,
      className: `${classes.richContainer} ${underline ? '' : classes.disableUnderline}`,
      onClick: this.focus,
      onFocus: this.focus,
      onKeyDown: () => {}
    }, React.createElement(Editor, {
      id: `${label}-richtextarea`,
      blockStyleFn: getBlockStyle,
      customStyleMap: getStyleMap(),
      editorState: editorState,
      handleKeyCommand: this.handleKeyCommand,
      onChange: this.onChange,
      onTab: this.onTab,
      onBlur: this.onBlur,
      ref: ref => {
        this.editor = ref;
      },
      spellCheck: true
    })), hasFocus && React.createElement("div", {
      className: classes.richControls
    }, React.createElement(BlockStyleControls, {
      t: t,
      classes: classes,
      editorState: editorState,
      onToggle: this.toggleBlockType
    }), React.createElement(InlineStyleControls, {
      t: t,
      classes: classes,
      editorState: editorState,
      onToggle: this.toggleInlineStyle
    })));
  }

}

function findLinkEntities(contentBlock, callback, contentState) {
  contentBlock.findEntityRanges(character => {
    const entityKey = character.getEntity();
    return entityKey !== null && contentState.getEntity(entityKey).getType() === 'LINK';
  }, callback);
}

function Link(props) {
  const {
    url
  } = props.contentState.getEntity(props.entityKey).getData();
  return React.createElement("a", {
    href: url,
    style: {
      color: 'green'
    },
    target: "_blank"
  }, props.children);
}

function findImageEntities(contentBlock, callback, contentState) {
  contentBlock.findEntityRanges(character => {
    const entityKey = character.getEntity();
    return entityKey !== null && contentState.getEntity(entityKey).getType() === 'IMAGE';
  }, callback);
}

function Image(props) {
  const {
    height,
    src,
    width
  } = props.contentState.getEntity(props.entityKey).getData();
  return React.createElement("img", {
    alt: "",
    src: src,
    height: height,
    width: width
  });
} // Custom overrides for "code" style.


function getStyleMap() {
  return {};
}

function getBlockStyle(block) {
  switch (block.getType()) {
    case 'blockquote':
      return 'RichEditor-blockquote';

    case 'justify-left':
      return 'RichEditor-blockquote';

    case 'justify-center':
      return 'RichEditor-blockquote';

    case 'justify-right':
      return 'RichEditor-blockquote';

    default:
      return null;
  }
}

export default withStyles(styles)(RichTextarea);